"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert = require("./assertions");
const convert = require("./conversions");
const validators_1 = require("./validators");
function propRef(prop, validator) {
    return `[${validator}]: ${prop}`;
}
exports.propRef = propRef;
/**
 * @description RFC 5322 based email address validation
 * @see https://stackoverflow.com/questions/201323/using-a-regular-expression-to-validate-an-email-address
 */
function Email() {
    return RegEx(/(?:[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*|"(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21\x23-\x5b\x5d-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])*")@(?:(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?|\[(?:(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9]))\.){3}(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9])|[a-z0-9-]*[a-z0-9]:(?:[\x01-\x08\x0b\x0c\x0e-\x1f\x21-\x5a\x53-\x7f]|\\[\x01-\x09\x0b\x0c\x0e-\x7f])+)\])/);
}
exports.Email = Email;
function ID() {
    return validators_1.Validator((input, prop) => {
        const ref = propRef(prop, ID.name);
        const value = convert.toInt(input);
        assert.isInt(value, ref);
        assert.isPositive(value, ref);
        return value;
    });
}
exports.ID = ID;
function RegEx(regEx) {
    return validators_1.Validator((input, prop) => {
        const ref = propRef(prop, RegEx.name);
        assert.isString(input, ref);
        assert.isRegEx(regEx, input, ref);
        return input;
    });
}
exports.RegEx = RegEx;
//# sourceMappingURL=extensions.js.map